#include <stdio.h>
#include <rpc/rpc.h>
#include <sys/time.h>
#include "../ahhead.h"
#include "wfdiscio_a.h"		/* uses a modified version of wfdiscio.h
				   to avoid conflicting structure defs */

#define MAXNAME	120
#define BUF_SZ	1024

char progname[10];

main(argc, argv)
/*
   converts an ah-format datafile to a
   CSS-format .wfdisc header file and a set
   of .w waveform files; dumps all files into
   the current directory (does not use the
   $WFPATH environment variable)

   February 1985, rbd

   Updated to expect xdr ah format input
   September 1989, Ken Howard
*/
int argc;
char *argv[];
{
	ahhed ah_head;
	XDR xdr_in;
	struct wfdisc wf_head;
	FILE *wfd_ptr;
	int wvf_fd;
	char arg[MAXNAME], prefix[MAXNAME];
	char wfd_name[MAXNAME], wvf_name[MAXNAME];
	float data_buf[BUF_SZ];
	int wv_count;
	long today, j, k, l;
	struct tm *now;
	struct timeval tp;
	struct timezone tzp;
	FILE *fopen();
	double htoe();
	int julian();
	struct tm *localtime();

	strcpy(progname, "ah2css");
	prefix[0]= '\0';

	for (argc--, argv++; argc; argc--, argv++) {
		strcpy(arg, *argv);
		if (!strncmp(arg, "-H", 2)) {
			fprintf(stderr, "usage: %s [-p output_file_prefix] < ah_input_file\n", progname);
			exit(0);
		}
		if (!strncmp(arg, "-p", 2)) {
			argc--;
			argv++;
			strcpy(prefix, *argv);
			if (strlen(prefix)+strlen(".wfdisc") > MAXNAME-1) {
				prefix[MAXNAME-1-strlen(".wfdisc")]= '\0';
				fprintf(stderr, "%s: prefix truncated to %s\n", progname, prefix);
			}
		}
		else {
			fprintf(stderr, "%s: unrecognized command option/argument\n", progname);
			exit(-1);
		}
	}

	xdrstdio_create(&xdr_in, stdin, XDR_DECODE);

	/* main loop -- converts one ah header/data pair per iteration */
	for (wv_count= 1; xdr_gethead(&ah_head, &xdr_in) == 1; wv_count++) {

		/* initialize wf_head with NULL values */
		wf_head= wfdisc_null;

		/* copy all pertinent fields of ah_head to wf_head */
		wf_head.date= (((long) ah_head.record.abstime.yr)*1000L)+((long) julian(&ah_head.record.abstime));
		if (wv_count == 1) {
			if (prefix[0] == '\0')
				sprintf(prefix, "%7ld", wf_head.date);
			strcpy(wfd_name, prefix);
			strcat(wfd_name, ".wfdisc");
			if ((wfd_ptr= fopen(wfd_name, "w")) == (FILE *) NULL) {
				fprintf(stderr, "%s: cannot open %s ... exiting.\n", progname, wfd_name);
				exit(-1);
			}
		}

		wf_head.tym= htoe(&ah_head.record.abstime);
		if (strncmp(ah_head.station.code, "null", 4))
			strcpy(wf_head.sta, ah_head.station.code);
		if (strncmp(ah_head.station.chan, "null", 4))
			strncpy(wf_head.chn, ah_head.station.chan, 4);
		wf_head.nsamp= ah_head.record.ndata;
		wf_head.smprat= 1./ah_head.record.delta;

		/********* calibr, calper are set to NULL **************/

		if (strncmp(ah_head.station.stype, "null", 4))
			strcpy(wf_head.instyp, ah_head.station.stype);
		strcpy(wf_head.segtyp, "o");
		switch (ah_head.record.type) {
		case FLOAT:
			strcpy(wf_head.dattyp, "f4");
			break;
		default:
			fprintf(stderr, "%s: unsupported wfdisc data type ... exiting.\n", progname);
			exit(-1);
			break;
		}

		/************ clip, chid are set to NULL **************/

		wf_head.wfid= (long) wv_count;
		sprintf(wf_head.file, "%s.%1ld.w", prefix, wf_head.wfid);
		wf_head.foff= 0L;
		if (!gettimeofday(&tp, &tzp)) {
			now= localtime(&((long) tp.tv_sec));
			wf_head.adate= (1900L+((long) now->tm_year))*1000L+((long) now->tm_yday);
		}
		strcpy(wf_head.remark, "ah2wf; ");

		/* output ASCII header info to .wfdisc file */
		fprintf(wfd_ptr, WFDISC_WCS, WFDISC_WVL(&wf_head));

		/* copy data from ah file to .w waveform file */
		if ((wvf_fd= creat(wf_head.file, 0755)) == -1) {
			fprintf(stderr, "%s: cannot create waveform file ... exiting.\n", progname);
			exit(-1);
		}
		if ((j= (long) write(wvf_fd, (char *) &ah_head.record.ndata, (unsigned) sizeof(long))) != sizeof(long)) {
			fprintf(stderr, "%s: data write error ... exiting.\n", progname);
			exit(-1);
		}
		for (j= ah_head.record.ndata*sizeof(float); j > 0; j-=k) {
			if (j > (long) (BUF_SZ*sizeof(float)))
				k= (long) (BUF_SZ*sizeof(float));
			else
				k= j;

			/* read data from ah file */
			if(!xdr_vector(&xdr_in, (char *) data_buf, (unsigned) k, sizeof(char), xdr_char)) {
				fprintf(stderr, "%s: data read error ... exiting.\n", progname);
				exit(-1);
			}

			/* output data to .w file */
			if ((l= (long) write(wvf_fd, (char *) data_buf, (unsigned) k)) != k) {
				fprintf(stderr, "%s: data write error ... exiting.\n", progname);
				exit(-1);
			}
		}
		close(wvf_fd);
	}
	fclose(wfd_ptr);
	xdr_destroy(&xdr_in);
}
