#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "wfdiscio.h"

#define	REG	register

#define	WFDISC_SIZE	209

char	*sprintf();

/*
** readin_wfdisc_file(wf_prefix)
**
**	Takes as an argument a prefix to a wfdisc file eg. "NTS".  It 
**      prepends the value of the WFPATH environment variable to the prefix
**      to locate the actual wfdisc file.  Upon locating the wfdisc file
**	it opens it and places a pointer to each wfdisc record it reads 
**	into an array of pointers to wfdisc records.
**
**      RETURNS: A pointer to an array of pointers to wfdisc structures.
** 		 NULL if there was an error.
**
**		from CSS, slightly altered by DCW 19 october 1985
*/
struct	wfdisc	**
readin_wfdisc_file(wf_prefix)
char *wf_prefix;
{
	static	struct	wfdisc	**wfdisc_mptr;
	static	char	string[80] = "?";
	REG	char	*station, *channel;
	REG	struct	wfdisc	**tfpp, *tfp;
	REG	int	count, num_rec;
	FILE		*fp;
	double		time;
	struct	stat	stat_buf;
	struct	wfdisc	tmp_wfd;

	char 	*db_name;
	char	*malloc(), *get_wfpath();

	/* If we've already figured this stuff out don't do it again. */
	if (wfdisc_mptr) return(wfdisc_mptr);

	/* finish up creating the name */
	db_name = malloc((unsigned) 256);
	sprintf(db_name, "%s.wfdisc", wf_prefix);
	
	/* Open the wfdisc file. */
	fp = fopen(db_name, "r");
	free(db_name);
	if (fp < 0)
	{
		fprintf(stderr, "readin_wfdisc: Couldn't open %s.wfdisc file.\n", wf_prefix );
		return( NULL );
	}

	/* Now we need to find out how big it is. */
	fstat(fp->_file, &stat_buf);
	
	/* If there are any bytes left over the file has problems. */
	if((stat_buf.st_size % WFDISC_SIZE) || (stat_buf.st_size == 0))
	{
		fprintf( stderr, "readin_wfdisc: Wrong size for %s.wfdisc file.\n", wf_prefix );
		return( NULL );
	}

	num_rec = stat_buf.st_size / WFDISC_SIZE;

	/* Allocate enough space for pointers to all the records. */
	if (!(tfpp = wfdisc_mptr =
		(struct wfdisc **) malloc((unsigned) (num_rec + 1)
				 * sizeof(struct wfdisc *))))
	{
		fprintf(stderr,"readin_wfdisc: No room for pointer array.\n");
		return( NULL );
	}

	count = 0;
	time = -999999.9;
	station = channel = string;
	tfp = &tmp_wfd;
	while(fscanf(fp, WFDISC_RCS, WFDISC_RVL(tfp)) != EOF )
	{	WFDISC_TRM(tfp);
		tfpp[count] = (struct wfdisc *)
			malloc((unsigned) sizeof(struct wfdisc));
		if( tfpp[count] == NULL )
		{
			fprintf(stderr,"read_wfdisc: Couldn't malloc %s.wfdisc record.\n", wf_prefix );
			return( NULL );
		}
		*tfpp[count] = tmp_wfd;	/* copy it */
		if(strcmp(station, tfpp[count]->sta) ||
				strcmp(channel, tfpp[count]->chan))
		{	station = tfpp[count]->sta;
			channel = tfpp[count]->chan;
			time = -999999.9;
		}
		else
		{	if(tfpp[count]->time < time)
			{	sprintf(string, "%s%0d%s",
					"readin_wfdisc: Record ", count,
					" not time ordered.");
				fprintf(stderr,"%s\n",string);
				return( NULL );
			}
			time = tfpp[count]->time;
		}
		count++;
	}

	tfpp[count] = NULL;

	fclose(fp);

	return(wfdisc_mptr);
}
