/*   This program converts nabalek ascii format data files (GDSN FORMAT)
 *     to "ah" format 
 *			Error conditions are;
 *					-1 = Error in command line input
 *					-2 = Error allocating space for data
 *					-3 = Error reading data from stdin
 *					-4 = Error writing data to stdout
 *
 *			Latest Revision: March 25, 1987
 *				by; Tom Boyd orginally written by Javier Pacheco

			Updated to write xdr ah format files
 *				by; Ken Howard September 26, 1989
 *
 */
#include <stdio.h>
#include <rpc/rpc.h>
#include <math.h>
#include <strings.h>
#include <ctype.h>
#include "../ahhead.h"

static int days_in_month[]= {31,28,31,30,31,30,31,31,30,31,30,31,31};

#define mod(a,b)	a - ((int)(a/b)) * b

char *progname;
main(argc,argv)
	int argc;
	char *argv[];
{
	float *data,*temp,three_db,junk;
	ahhed head;
	XDR xdr_out;
	char *calloc(),buffer[12],string[12];
	int i,j,count,filtered,jul;
	
	progname=argv[0];
	if((out_is_tty() != 0)||(in_is_tty()!=0))
	{
		fprintf(stderr,"Usage: %s  < stdin > stdout\n\n",progname);
				exit(-1);
	}
	

	/* create xdr stream for stdout */
	xdrstdio_create(&xdr_out, stdout, XDR_ENCODE);

	get_null_head(&head);

	while(fscanf(stdin, "%s %s", head.station.code, head.station.chan) != EOF)
	{

		if((temp=(float *)calloc(100000,sizeof(float))) == NULL)
		{
			fprintf(stderr, "ERROR allocating space in %s", progname);
			exit(-2);
		}
		fscanf(stdin,"%f %f %f", &head.station.slat, &head.station.slon, &head.station.elev);
		fscanf(stdin,"%hd %d",&head.record.abstime.yr, &jul);
				/* next field may have blanks */
			for(i=0;i<11;i++)buffer[i]=getchar();
			strncpy(string,&buffer[0],11);
			string[3]='\0';
			sscanf(string,"%2hd",&head.record.abstime.hr);
			strncpy(string,&buffer[3],11);
			string[2]='\0';
			sscanf(string,"%2hd",&head.record.abstime.mn);
			strncpy(string,&buffer[5],11);
			sscanf(string,"%f",&head.record.abstime.sec);

		fscanf(stdin,"%f", &head.record.delta);
		head.record.delta=1./head.record.delta;
		fscanf(stdin,"%e %e", &head.station.DS, &head.station.A0);
		month_day(&head.record.abstime, jul);
		fscanf(stdin,"%f", &head.station.cal[0].pole.r);
		for(i=1;i <= (int)head.station.cal[0].pole.r; i++)
				fscanf(stdin,"%e %e", &head.station.cal[i].pole.r, &head.station.cal[i].pole.i);
		fscanf(stdin,"%f", &head.station.cal[0].zero.r);
		for(i=1;i <= (int)head.station.cal[0].zero.r; i++)
				fscanf(stdin,"%e %e", &head.station.cal[i].zero.r, &head.station.cal[i].zero.i);
		fscanf(stdin,"%d %e %e", &filtered, &three_db, &junk);
		j=0;
		fscanf(stdin,"%s", buffer);
		while((buffer[1] != 'x'))
		{
			if((isdigit(buffer[0])) || (isdigit(buffer[1])) )
			{
		 		temp[j] = atof(buffer);
				j++;
			}
			fscanf(stdin,"%s", buffer);
		}
		head.record.ndata=j;
		head.record.log[0]='\0';
		logger("nabel;",&head);
		head.record.type=(short)FLOAT;
		if(xdr_puthead(&head, &xdr_out) != 1)
		{
			fprintf(stderr, "%s: error writing ah header\n", progname);
			exit(-4);
		}

		maxamp(&head,temp);

		if(xdr_putdata(&head,(char *)temp, &xdr_out) < 0)
		{
			fprintf(stderr, "%s: error writing ah data\n", progname);
			exit(-4);
		}
		cfree((char *)temp);
	}
	xdr_destroy(&xdr_out);
}


month_day(t, jul_day)
register struct ah_time *t;
int jul_day;
{
	int i, dim, leap;
	int isleap();

	leap= isleap(t->yr);
	t->day= jul_day;
	for (i= 0; i < 12; i ++) {
		dim= days_in_month[i];
		if (leap && (i == 1)) dim++;
		if (t->day <= dim ) break;
		t->day-= dim;
	}
	t->mo= i+1;
	return;
}


isleap(year)
int year;
{
	return(year % 4 == 0 && year % 100 != 0 || year % 400 == 0);
}
