#include <stdio.h>
#include <rpc/rpc.h>

#include "muxhead.h"
#include "ahhead.h"

#define MAXNAME	80
#define MAXSTA	80
#define STDOUT	1
#define BUF_SZ	4096

#define boolean int
#ifdef NOVOID
#define void	int
#endif

char progname[20];
char template_head[25]; 
char template_data[25];
ahhed head;
boolean split= 0;	/* split output into files containing
			   an equal number of data records? */
boolean stations= 0;	/* additional info in station file? */
boolean gain= 0;	/* additional info in gain file? */
boolean no_massage= 0;	/* if data is to remain on the PDP, don't
			   convert binary numbers to Sun format */
boolean was_pdp= 0;	/* if data was from pdp archive		*/
boolean was_intel= 0;	/* if data is from intel		*/

int rec_lmt, rec_cnt;
static char template[15];

int out_file;		/* output file -- defaults to STDOUT
			   unless the split flag has been set */
FILE *pick, *stn, *gn;	/* pick, station and gain files */
FILE *resp;		/* response file		*/

short buf[BUF_SZ];	/* buffer of original integer data */
typedef union u_tag {	/* can't do bit-twiddling on floats, so  */
	long ln;	/* we have to pretend that they're longs */
	float fl;
} jk_hyde;
jk_hyde jk_buf[BUF_SZ];	/* conversion buffer */

main(argc, argv)
int argc;
char *argv[];
{
	char arg[MAXNAME];
	char st_fname[MAXNAME], gn_fname[MAXNAME], resp_fname[MAXNAME];
	char header_name[MAXNAME], data_name[MAXNAME], pick_name[MAXNAME];
	int err;
	void help();

	strcpy(progname, "ping2ah");
	if (argc < 2) {
		fprintf(stderr, "bad usage: try %s [-Help] \n", progname);
		exit(1);
	}

	out_file= STDOUT;

	/* main loop -- processes one argument
	   (or option/argument pair) at a time */
	for (argc--, argv++; argc; argc--, argv++) {
		strcpy(arg, *argv);
		if (!strncmp(arg, "-H", 2)) {
			help();
			continue;
		}
		if (!strncmp(arg, "-s", 2)) {
			argc--;
			argv++;
			stations= 1;
			strcpy(st_fname, *argv);
			if ((stn= fopen(st_fname, "r")) == (FILE *) NULL) {
				fprintf(stderr, "%s: cannot open station file ... ignoring -s option\n", progname);
				stations= 0;
			}
			continue;
		}
		if (!strncmp(arg, "-c", 2)) {
			argc--; argv++;
			strcpy(gn_fname, *argv);

			argc--; argv++;
			strcpy(resp_fname, *argv);
			gain= 1;
			if ((gn= fopen(gn_fname, "r")) == (FILE *) NULL) {
				fprintf(stderr, "%s: cannot open gain file ... ignoring -c option\n", progname);
				gain= 0;
			}
			if ((resp= fopen(resp_fname, "r")) == (FILE *) NULL) {
				fprintf(stderr, "%s: cannot open response file ... ignoring -c option\n", progname);
				gain= 0;
			}
			continue;
		}
		if (!strncmp(arg, "-x", 2)) {
			if(was_pdp) {
				fprintf(stderr, "%s: cannot use both intel and pdp flags...ignoring intel flag\n");
				continue;
			}
			convert2ping(argv[argc - 1], "intel");
			was_intel = 1;
			no_massage= 1;
			continue;
		}

		if (!strncmp(arg, "-y", 2)) {
			if(was_intel) {
				fprintf(stderr, "%s: cannot use both intel and pdp flags...ignoring pdp flag\n");
				continue;
			}
			was_pdp = 1;
			convert2ping(argv[argc - 1], "pdp");
			no_massage= 1;
			continue;
		}

		if (!strncmp(arg, "-r", 2)) {
			argc--;
			argv++;
			split= 1;
			rec_lmt= atoi(*argv);
			rec_cnt= 0;
			strcpy(template, "ah.XXXXXX");
			mktemp(template);
			if ((out_file= creat(template, 0644)) == -1) {
				fprintf(stderr, "%s: cannot open output file ... writing to stdout\n", progname);
				out_file= STDOUT;
				split= 0;
			}
			continue;
		}
		else if (!strncmp(arg, "-n", 2)) {
			no_massage= 1;
			continue;
		}
		else {
			strcpy(header_name, arg);
			strcat(header_name, "D");
			strcpy(data_name, arg);
			strcat(data_name, "d");
			strcpy(pick_name, arg);
			strcat(pick_name, "p");

			if(was_pdp || was_intel)
				err = convert(template_head, template_data, pick_name, out_file);
			else
				err = convert(header_name, data_name, pick_name, out_file);
			if (err)
				fprintf(stderr, "%s: cannot convert %s\n", progname, arg);
		}
	}
	/* remove template files */
	if(was_pdp || was_intel) {
		unlink(template_head);
		unlink(template_data);
	}
}

void
help()
/* prints out a detailed help message */
{
	fprintf(stderr, "\n%s:  converts ping datafiles into ah format\n", progname);
	fprintf(stderr, "          (each argument should be the common\n");
	fprintf(stderr, "          prefix of a header_file/data_file pair).\n");
	fprintf(stderr, "\nUsage: %s [options] common_prefix > output_name\n", progname);
	fprintf(stderr, "\noptions:\n");
	fprintf(stderr, "   -s st	read additional information from station_file st;\n");
	fprintf(stderr, "   -c cha resp	read additional information from channels file cha and response_file resp;\n");
	fprintf(stderr, "   -r n  	split the output into files containing n data records each;\n");
	fprintf(stderr, "   -n     	don't convert to Sun binary formats.\n");
	fprintf(stderr, "   -x    	if data is in intel format.\n");
	fprintf(stderr, "   -y		if data is in pdp archive format.\n\n");
	fprintf(stderr, "Note: A pickfile named common_prefixp must be present in the same\n");
	fprintf(stderr, "      directory as the data if event information is to be added to ah files.\n");
	fprintf(stderr, "\n");
}

int
convert(header, data, pickfile, out)
/* converts one header_file/data_file pair to ah binary format */
char header[], data[], pickfile[];
int out;
{
	XDR xdr_out;
	struct muxhead mux;
	struct stahead sta;
	struct event_info evt;
	int i;
	int h, d;
	int date[5];
	short *ptr;
	jk_hyde *jptr;
	long j, k, l, m;
	float f_sec;
	FILE *fopen(), *fildes;
	char *calloc();
	long lseek();
	void get_pick(), copy_pick(), get_sta(), get_gain();
	void null_hd(), sun_hd(), sun_float();


	/* convert output file descripter to file pointer */
	fildes = fdopen(out, "w");
	xdrstdio_create(&xdr_out, fildes, XDR_ENCODE);

	/* open ping header and data files */
	if ((h= open(header, 0)) == -1) {
		fprintf(stderr, "%s: cannot open header_file %s\n", progname, header);
		return(1);
	}
	if ((d= open(data, 0)) == -1) {
		fprintf(stderr, "%s: cannot open data_file %s\n", progname, data);
		close(h);
		return(1);
	}
	if ((pick= fopen(pickfile, "r")) == (FILE *) NULL)
		fprintf(stderr, "%s: cannot open pick file %s\n", progname, pickfile);
	else
		get_pick(&evt);

	/* read header file */
	if (read(h, (char *) &mux, sizeof(mux)) < sizeof(mux)) {
		fprintf(stderr, "%s: cannot read header_file %s\n", progname, header);
		close(h);
		close(d);
		fclose(pick);
		fclose(fildes);
		xdr_destroy(&xdr_out);
		return(1);
	}
		

	/* loop once for each data record to be written */
	for (i= 0; i < mux.nchan; i++) {

		/* open a new output file? */
		if (split) {
			if (++rec_cnt > rec_lmt) {
				rec_cnt= 1;
				close(out_file);
				strcpy(template, "ah.XXXXXX");
				mktemp(template);
				if ((out_file= creat(template, 0644)) == -1) {
					fprintf(stderr, "%s: cannot open output file ... writing to stdout\n", progname);
					out_file= STDOUT;
					split= 0;
				}
				else
					out= out_file;
			}
		}

		/* get station info */
		if (read(h, (char *) &sta, sizeof(sta)) < sizeof(sta)) {
			fprintf(stderr, "%s: cannot read header_file %s\n", progname, header);
			close(h);
			close(d);
			fclose(pick);
			fclose(fildes);
			xdr_destroy(&xdr_out);
			return(1);
		}

		/* set header fields */
		null_hd(&head);
		strcpy(head.station.code, sta.name);
		strcpy(head.record.rcomment, mux.comment);
		head.record.type= FLOAT;
		head.record.ndata= mux.length;
		head.record.delta= 1000./(float) mux.lrate;
		mingrg(date, &mux.lmin);
		head.record.abstime.yr= (short) date[0];
		head.record.abstime.mo= (short) date[1];
		head.record.abstime.day= (short) date[2];
		head.record.abstime.hr= (short) date[3];
		head.record.abstime.mn= (short) date[4];
		f_sec= (float) mux.lsec;
		f_sec/= (float) 1000000.;
		head.record.abstime.sec= f_sec;

		/* more info in pick, station or gain files? */
		if (pick != (FILE *) NULL)
			copy_pick(&evt, &head);
		if (stations)
			get_sta(&head);
		if (gain)
			get_gain(&head);

		/* convert header to Sun format */
		if (!no_massage)
			sun_hd(&head);

		/* write out header */
		if(xdr_puthead(&head, &xdr_out) != 1) {
			fprintf(stderr, "%s: header write error\n", progname);
			close(h);
			close(d);
			fclose(pick);
			fclose(fildes);
			xdr_destroy(&xdr_out);
			return(1);
		}

		/* locate start of data record */
		if ((j= lseek(d, ((long) i)*mux.length*2L, 0)) < 0) {
			fprintf(stderr, "%s: cannot seek through data_file %s\n", progname, data);
			close(h);
			close(d);
			fclose(pick);
			fclose(fildes);
			xdr_destroy(&xdr_out);
			return(1);
		}

		/* perform data conversion */
		for (k= mux.length*(long) sizeof(short); k; k-= j) {
			if (k >= (long) (BUF_SZ*sizeof(short)))
				l= (long) (BUF_SZ*sizeof(short));
			else
				l= k;
			if ((j= (long) read(d, (char *) buf, (unsigned) l)) < l) {
				fprintf(stderr, "%s: cannot read data_file %s\n", progname, data);
				close(h);
				close(d);
				fclose(pick);
				fclose(fildes);
				xdr_destroy(&xdr_out);
				return(1);
			}


			m= j/sizeof(short);
			for (l= 0, ptr= buf, jptr= jk_buf; l < m; l++, ptr++, jptr++)
				jptr->fl= (float) *ptr;

			/* convert data to Sun floating point format */
			if (!no_massage)
				sun_float(jk_buf, (int) m);

			/* write data */
			if(!xdr_vector(&xdr_out, (char *) jk_buf, m, sizeof(float), xdr_float)) {
				fprintf(stderr, "%s: data write error\n", progname);
				close(h);
				close(d);
				fclose(pick);
				fclose(fildes);
				xdr_destroy(&xdr_out);
				return(1);
			}
		}
	}

	close(h);
	close(d);
	fclose(pick);
	fclose(fildes);
	xdr_destroy(&xdr_out);
	return(0);
}

void
null_hd(hd)
/* clear a header structure */
ahhed *hd;
{
	int i;

	strcpy(hd->station.code, "null");
	strcpy(hd->station.chan, "null");
	strcpy(hd->station.stype, "null");
	hd->station.slat= 0.0;
	hd->station.slon= 0.0;
	hd->station.elev= 0.0;
	hd->station.DS= 0.0;
	hd->station.A0= 0.0;
	for (i= 0; i< NOCALPTS; i++) {
		hd->station.cal[i].pole.r= 0.0;
		hd->station.cal[i].pole.i= 0.0;
		hd->station.cal[i].zero.r= 0.0;
		hd->station.cal[i].zero.i= 0.0;
	}

	hd->event.lat= 0.0;
	hd->event.lon= 0.0;
	hd->event.dep= 0.0;
	hd->event.ot.yr= (short) 0;
	hd->event.ot.mo= (short) 0;
	hd->event.ot.day= (short) 0;
	hd->event.ot.hr= (short) 0;
	hd->event.ot.mn= (short) 0;
	hd->event.ot.sec= 0.0;
	strcpy(hd->event.ecomment, "null");

	hd->record.type= (short) 0;
	hd->record.ndata= 0L;
	hd->record.delta= 0.0;
	hd->record.maxamp= 0.0;
	hd->record.abstime.yr= (short) 0;
	hd->record.abstime.mo= (short) 0;
	hd->record.abstime.day= (short) 0;
	hd->record.abstime.hr= (short) 0;
	hd->record.abstime.mn= (short) 0;
	hd->record.abstime.sec= 0.0;
	hd->record.rmin= 0.0;
	strcpy(hd->record.rcomment, "null");
	strcpy(hd->record.log, "null");

	for (i= 0; i< NEXTRAS; i++)
		hd->extra[i]= 0.0;

	return;
}

void
get_pick(ev)
/* reads additional information into
   the header from a pick file */
struct event_info *ev;
{
	char line[130], tmp[7];
	int lines;
	char *fgets();
	int atoi();
	double atof();

	/* we're only interested in the last line, but we can't
	   tell which is the last line until we've read past it,
	   so count the lines, then start again from the top
	   (there must be a better way to do this!!)	      */
	fseek(pick, 0L, 0);
	while (1) {
		if (fgets(line, 129, pick) == (char *) NULL)
			break;
		else
			lines++;
	}
	fseek(pick, 0L, 0);
	for (; lines > 0; lines--)
		fgets(line, 129, pick);

	strncpy(tmp, line, 2);
	tmp[2]= '\0';
	ev->ot.yr= ((short) atoi(tmp))+((short) 1900);
	strncpy(tmp, line+2, 2);
	tmp[2]= '\0';
	ev->ot.mo= (short) atoi(tmp);
	strncpy(tmp, line+4, 2);
	tmp[2]= '\0';
	ev->ot.day= (short) atoi(tmp);
	strncpy(tmp, line+6, 2);
	tmp[2]= '\0';
	ev->ot.hr= (short) atoi(tmp);
	strncpy(tmp, line+8, 2);
	tmp[2]= '\0';
	ev->ot.mn= (short) atoi(tmp);
	strncpy(tmp, line+10, 4);
	tmp[4]= '\0';
	ev->ot.sec= ((float) atof(tmp))/100.;
	strncpy(tmp, line+14, 2);
	tmp[2]= '\0';
	ev->lat= (float) atof(tmp);
	strncpy(tmp, line+17, 4);
	tmp[4]= '\0';
	ev->lat+= ((float) atof(tmp))/(60.*100.);
	if ((line[16] == 's') || (line[16] == 'S'))
		ev->lat= -(ev->lat);

	strncpy(tmp, line+21, 3);
	tmp[3]= '\0';
	ev->lon= (float) atof(tmp);
	strncpy(tmp, line+25, 4);
	tmp[4]= '\0';
	ev->lon+= ((float) atof(tmp))/(60.*100.);
	if ((line[24] == 'w') || (line[24] == 'W') || (line[24] == ' '))
		ev->lon= -(ev->lon);

	strncpy(tmp, line+29, 5);
	tmp[5]= '\0';
	ev->dep= ((float) atof(tmp))/100.;
}

void
copy_pick(ev, hd)
/* copy the info we've read from the 
   pick file into the new header  */
struct event_info *ev;
ahhed *hd;
{
	hd->event.lat= ev->lat;
	hd->event.lon= ev->lon;
	hd->event.dep= ev->dep;
	hd->event.ot.yr= ev->ot.yr;
	hd->event.ot.mo= ev->ot.mo;
	hd->event.ot.day= ev->ot.day;
	hd->event.ot.hr= ev->ot.hr;
	hd->event.ot.mn= ev->ot.mn;
	hd->event.ot.sec= ev->ot.sec;
}

void
get_sta(hd)
/* reads additional information into
   the header from a station file */
ahhed *hd;
{
	char line[130], tmp[7];
	int i;
	char *fgets();
	double atof();

	fseek(stn, 0L, 0);
	while (1) {
		if (fgets(line, 129, stn) == (char *) NULL) {
			fprintf(stderr, "%s: station %s not present in station file\n", progname, hd->station.code);
			break;
		}

		/* here with a matching name */
		else if (!strncmp(line, hd->station.code, 4)) {

			/* get rid of the newline read by fgets() */
			for (i= 0; i < 129; i++)
				if (line[i] == '\n') {
					line[i]= '\0';
					break;
				}

			/* copy all pertinent info into the new header */
			strncpy(tmp, line+5, 2);
			tmp[2]= '\0';
			hd->station.slat= (float) atof(tmp);
			strncpy(tmp, line+8, 5);
			tmp[5]= '\0';
			hd->station.slat+= ((float) atof(tmp))/60.;
			if ((line[13] == 's') || (line[13] == 'S'))
				hd->station.slat= -(hd->station.slat);

			strncpy(tmp, line+14, 3);
			tmp[3]= '\0';
			hd->station.slon= (float) atof(tmp);
			strncpy(tmp, line+18, 5);
			tmp[5]= '\0';
			hd->station.slon+= ((float) atof(tmp))/60.;
			if ((line[23] == 'w') || (line[23] == 'W'))
				hd->station.slon= -(hd->station.slon);

			strncpy(tmp, line+24, 4);
			tmp[4]= '\0';
			hd->station.elev= (float) atof(tmp);

			strncpy(hd->station.chan, line+66, 4);
			hd->station.chan[4]= '\0';

			break;
		}
	}
}

void
get_gain(hd)
/* reads additional information into
   the header from a gain file and a
   response file
*/
ahhed *hd;
{
	char line[120];
	boolean found;
	float vco_gain;
	char seis_type[5];

	/* try to find station in gain file */
	found = 0;
	fseek(gn, 0L, 0);
	while((fgets(line, 120, gn)) != NULL) {

		/* here with match of station on gain file */
		if(!strncmp(line+4, hd->station.code, 4)) {
			found = 1;
			/* get vco_gain and seismometer type */
			sscanf(line+32, "%f%s", &vco_gain, seis_type);
			seis_type[3] = '\0';
			break;
		}
	}

	/* fill ah structure with additional info */
	if(found) {
		poles_and_zeros(hd, seis_type[1]);
		calculate_gain(seis_type, vco_gain, &(hd->station.DS));

		switch(seis_type[1]) {
		case 'n':
		case 'g':
		case 'o':
			strcpy(hd->station.chan,"sp");
			strcpy(hd->station.stype,"HS10");
			break;
		case 'b':
			strcpy(hd->station.chan,"bs");
			strcpy(hd->station.stype,"CMG4");
			break;
		case 'i':
			strcpy(hd->station.chan,"bi");
			strcpy(hd->station.stype,"CMG4");
			break;
		case 'f':
			strcpy(hd->station.chan,"fb");
			strcpy(hd->station.stype,"FBA");
			break;
		default:
			strcpy(hd->station.chan," ");
			strcpy(hd->station.stype, " ");
			break;
		}

		hd->station.chan[2] = hd->station.code[3];
		hd->station.chan[3] = '\0';
	}
	else
		fprintf(stderr, "ping2ah: cannot find station %s in gain file.\n", hd->station.code);
}



/* Floating point formats --

	Floating point formats for the 11/70 and the Sun are similar in that
each consists of 32 bits (four bytes), with the high-order bit representing
the sign (1 = negative), the next eight bits the exponent, and the remaining
23 low-order bits the mantissa. Mantissa formats are identical -- only the
exponent format differs. The value of any given floating point number on either
machine will be
		(-s) * (2**e) * (1 + m)
where s is the sign bit, e is the true exponent (see below), and m is the
mantissa. Computation of the mantissa is best shown by example: a mantissa of
01011 ... (high-order bit to the left) is equal to
	0*(2**-1) + 1*(2**-2) + 0*(2**-3) + 1*(2**-4) + 1*(2**-5) + ...

	The 11/70 exponent is in excess 0201 format (you must subtract 0201
from the contents of the 8-bit exponent to obtain the true exponent), while the
Sun exponent is in excess 0177 format. In addition to adjusting for this
difference, adjacent bytes must be swapped when transferring between these
two machines. Thus, to convert -1.5 from 11/70 format to Sun format:

	(1) 1 10000001 10000000000000000000000   (original 11/70 format)
	    s exponent        mantissa

	(2) 1 01111111 10000000000000000000000   (subtract 2 from exponent)
	    s exponent        mantissa

	(3) 10111111 11000000 00000000 00000000  (swap adjacent bytes)
			     |
			     V
	    11000000 10111111 00000000 00000000  (send this to the Sun)

	(When this number is examined on the Sun, it will look like (2) above.)
*/

#define EXP_START	23
#define BIAS_DIFF	02L

void
sun_float(jkb, length)
/* convert a contiguous array of length
   jk_hydes to Sun floating point format */
jk_hyde *jkb;
int length;
{
	long e;
	int len;
	jk_hyde *j;
int n;

	len= length;
	for (j= jkb; len; len--, j++) {

		/* get original exponent */
		e= (j->ln >> EXP_START) & 0377L;

		/* compute new exponent */
		switch ((int) e) {
		case 0:
			break;
		default:
			e-= BIAS_DIFF;
			break;
		}

		/* mask out original exponent and
		   OR with the new exponent    */
		j->ln= (j->ln & 020037777777L) | (e << EXP_START);
	}

	/* now swap bytes */
	swab((short *) jkb, (short *) jkb, length*4);
}

void
sun_hd(hd)
/* convert all integer and float fields
   of an ah header to Sun format */
ahhed *hd;
{
	int i, flt_size;
	jk_hyde jk[25];
	void sun_float();

	flt_size= (int) sizeof(float);
	jk[0].fl= hd->station.slat;
	jk[1].fl= hd->station.slon;
	jk[2].fl= hd->station.elev;
	jk[3].fl= hd->station.DS;
	jk[4].fl= hd->station.A0;
	jk[5].fl= hd->event.lat;
	jk[6].fl= hd->event.lon;
	jk[7].fl= hd->event.dep;
	jk[8].fl= hd->event.ot.sec;
	jk[9].fl= hd->record.delta;
	jk[10].fl= hd->record.maxamp;
	jk[11].fl= hd->record.abstime.sec;
	jk[12].fl= hd->record.rmin;

	sun_float(jk, 13);
	bcopy((char *) &(jk[0].fl), (char *) &(hd->station.slat), flt_size);
	bcopy((char *) &(jk[1].fl), (char *) &(hd->station.slon), flt_size);
	bcopy((char *) &(jk[2].fl), (char *) &(hd->station.elev), flt_size);
	bcopy((char *) &(jk[3].fl), (char *) &(hd->station.DS), flt_size);
	bcopy((char *) &(jk[4].fl), (char *) &(hd->station.A0), flt_size);
	bcopy((char *) &(jk[5].fl), (char *) &(hd->event.lat), flt_size);
	bcopy((char *) &(jk[6].fl), (char *) &(hd->event.lon), flt_size);
	bcopy((char *) &(jk[7].fl), (char *) &(hd->event.dep), flt_size);
	bcopy((char *) &(jk[8].fl), (char *) &(hd->event.ot.sec), flt_size);
	bcopy((char *) &(jk[9].fl), (char *) &(hd->record.delta), flt_size);
	bcopy((char *) &(jk[10].fl), (char *) &(hd->record.maxamp), flt_size);
	bcopy((char *) &(jk[11].fl), (char *) &(hd->record.abstime.sec), flt_size);
	bcopy((char *) &(jk[12].fl), (char *) &(hd->record.rmin), flt_size);

	for (i= 0; i < NOCALPTS; i++) {
		jk[0].fl= hd->station.cal[i].pole.r;
		jk[1].fl= hd->station.cal[i].pole.i;
		jk[2].fl= hd->station.cal[i].zero.r;
		jk[3].fl= hd->station.cal[i].zero.i;
		sun_float(jk, 4);
		bcopy((char *) &(jk[0].fl), (char *) &(hd->station.cal[i].pole.r), flt_size);
		bcopy((char *) &(jk[1].fl), (char *) &(hd->station.cal[i].pole.i), flt_size);
		bcopy((char *) &(jk[2].fl), (char *) &(hd->station.cal[i].zero.r), flt_size);
		bcopy((char *) &(jk[3].fl), (char *) &(hd->station.cal[i].zero.i), flt_size);
	}

	for (i= 0; i < NEXTRAS; i++)
		jk[i].fl= hd->extra[i];
	sun_float(jk, NEXTRAS);
	for (i= 0; i < NEXTRAS; i++)
		bcopy((char *) &(jk[i].fl), (char *) &(hd->extra[i]), flt_size);

	swab(&hd->event.ot.yr, &hd->event.ot.yr, 10);
	swab(&hd->record.type, &hd->record.type, 6);
	swab(&hd->record.abstime.yr, &hd->record.abstime.yr, 10);
}
