#include <stdio.h>
#include <math.h>
#include "ahhead.h"

#define TRUE		1
#define FALSE		0

extern FILE *resp;

poles_and_zeros(h, type)
/* read poles, zeros and normalization from respose file */
ahhed *h;   /* ah header 	*/
char type; /* seismometer type 	*/
{
	char *fgets(), line[120];
	char seis_type[5];	/* must match incoming type */
	int i;

	fseek(resp, 0L, 0);
	while((fgets(line, 120, resp) != NULL)) {
		/* found type in response file */
		if(!strncmp(line, "type:", 5)) {
			sscanf(line+strlen("type:	"), "%c", seis_type);
			/* type matches incoming type */
			if(seis_type[0] == type) {
				fgets(line, 120, resp);
				sscanf(line+strlen("normalization:	"), "%f", &(h->station.A0));

				/* skip two lines */
				fgets(line, 120, resp);
				fgets(line, 120, resp);
				for(i = 0; i < NOCALPTS; i++) {
					fgets(line, 120, resp);
					sscanf(line, "%f%f%f%f", &(h->station.cal[i].pole.r), &(h->station.cal[i].pole.i),
							   &(h->station.cal[i].zero.r), &(h->station.cal[i].zero.i));
				}
				break;
			}
		}
	}
}

calculate_gain(type, vco_gain, gain)
/* calculate gain from vco gain and TSS gain */
char type[]; /* seismometer type */
float vco_gain;
float *gain; /* return value */
{
	char line[200];
	char seis_type[5];		/* must match incoming type */
	int found;
	float gain_a, gain_b, gain_c;	/* three possible gain system gain values for each type */
	float factor;			/* analog tss gain is a constant factor of digital gain */
	float tss_gain;
	double atof(), pow(), exponent;

	found = FALSE;
	fseek(resp, 0L, 0);
	while((fgets(line, 120, resp) != NULL)) {
		/* found type in response file */
		if(!strncmp(line, "type:", 5)) {
			sscanf(line+strlen("type:	"), "%c", seis_type);
			/* type matches incoming type */
			if(seis_type[0] == type[1]) {
				fgets(line, 120, resp);
				fgets(line, 120, resp);
				sscanf(line+strlen("system gain:	"), "%f%f%f", &gain_a, &gain_b, &gain_c);
				found = TRUE;
				break;
			}
		}
	}

	if(!found || vco_gain == 1.0)
		*gain = 0.0;
	else {
		/* find which of three columns is appropriate gain */
		if(strlen(type) == 2) /* first column */
			tss_gain = gain_a;
		else if(type[2] == 'b') /* second column */
			tss_gain = gain_b;
		else if(type[2] == 'c') /* third column */
			tss_gain = gain_c;


		/* gain formula - uses a factor of 3/4 going from digital to analog */
		if(type[0] == '1')
			factor = 1.0;
		if(type[0] == 't')
			factor = .75;
		exponent = (vco_gain - (double) 48.)/(double) 20.;
		*gain = (float) (tss_gain * factor * pow((double) 10., exponent));
	}
}
