/*
	Program SEGY2ah

	Program takes modified SEGy files as produced by 
	pas2SEGY program from PASSCAL data on EXABYTE tapes
	and reformats them into an AH data file.

	The SEGY modification is as follows:
			trhdr[100] = MSB of sample rate in microseconds
			trhdr[101] = LSB of sample rate in microseconds
			trhdr[102]=1 ==> 32 bit integer data
			trhdr[102]=0 ==> 16 bit integer data
			trhdr[103] = start of trace in milliseconds
			trhdr[104] = trigger time year
			trhdr[105] = trigger time julian day
			trhdr[106] = trigger time hour
			trhdr[107] = trigger time minutes
			trhdr[108] = trigger time seconds
			trhdr[109] = trigger time  milliseconds
			trhdr[110] = MSW of scale factor
			trhdr[111] = LSW of scale factor

	Written by Tim Ahern, IRIS
		August 9, 1988

	modified TK Ahern, IRIS
		8/31/88
		insert event trigger time into ah header

	modified JC Fowler 
		1/2/89
		insert scale factor info and correct position
		calculations.
*/


#include <stdio.h>	/* Standard I/O header file			*/
#include <ctype.h>
#include <string.h>
#include <fcntl.h>
#include <math.h>
#include "ahhead.h"


char *progname;
main (argc, argv)


int		argc;
char		*argv[];

{
   FILE	*fpin, *fpout;		/* Pointer to the input  and output files */
   ahhed   ahhead;              /* structure definead in ahhead.h   */ 
   struct time ttime;

   
   int i, j, int_hold, num_sam, sam_rate, ifile;
   short trhdr[120], short_hold, gain ;
   unsigned char *cptr, *cptr1, *cptr2 ;
   unsigned char FileIN[64] , FileOUT[64];
   float dt, *data, *dout;
   float xlat, xlon, xs, scale ;
   long	  llat, llon ;
   char *calloc(), *datac; 
   void cfree();
   static int day_tab[2][13] ={
      {0,31,28,31,30,31,30,31,31,30,31,30,31},
      {0,31,29,31,30,31,30,31,31,30,31,30,31}
   };
   int leap, iyear, yearday ;

   /* take command line arguments as filesnames and flags or
      ask for the filename to be used for the conversion      */

   progname = argv[0];

   for (i=0 ; i<64 ; i++) FileIN[i] = 0;
   if ( argc == 1 ){
      printf("\n Usage %s file_name(s) \n",progname);
      exit(1); 
   }
for(ifile=1; ifile < argc; ifile++){
      strcpy( FileIN, &argv[ifile][0] );
      if( strcmp(FileIN,"ah") == 0) continue; /* don't work on prev. directory */
      if( strcmp(FileIN,"sac") == 0) continue; /* don't work on prev. directory */
      printf(" %s \n",FileIN);

         /* open the input file and go to the output directory	*/

   if (( fpin = fopen(FileIN, "rb")) == NULL ){
      printf( "\n ******  there has been an error opening %s for input  *****", FileIN);
      fclose(fpin);
      exit(1);
   }

   /*  start reading the data file   */
   /*  first read the SEGY trace header    */

   cptr = ( unsigned char *) &trhdr[0];
   for ( i= 0; i< 240 ; i++){
      *cptr = fgetc(fpin);
      cptr++;
   }
   num_sam = trhdr[57];
   if (trhdr[58] == 1){
      sam_rate = trhdr[100] * 32768 + trhdr[101];
   }else{
      sam_rate = trhdr[58];
   }
   dt = ( float) sam_rate / 1000000.0;

   /* fill in the ah header   */

   get_null_head(&ahhead);

   /* now the record information   */

   ahhead.record.type = FLOAT;
   ahhead.record.ndata = trhdr[57];
   ahhead.record.delta = dt;
   ahhead.record.maxamp = 1;

   ahhead.record.abstime.yr = trhdr[78];

   iyear = trhdr[78];
   if(iyear%4 == 0){
      leap=1;
   }else{
      leap=0;
   }
   yearday = trhdr[79];
   for(iyear=1; yearday > day_tab[leap][iyear]; iyear++)yearday -= day_tab[leap][iyear];

   ahhead.record.abstime.mo = iyear;
   ahhead.record.abstime.day = yearday; 
   ahhead.record.abstime.hr = trhdr[80];
   ahhead.record.abstime.mn = trhdr[81];
   ahhead.record.abstime.sec = trhdr[82] + trhdr[103]/1000.0;
   ahhead.record.rmin = ahhead.record.abstime.sec;

   /* now the station information  */

   ahhead.station.code[0]= FileIN[9];
   ahhead.station.code[1]= FileIN[10];
   ahhead.station.code[2]= FileIN[11];
   ahhead.station.code[3]= FileIN[12];
   sprintf(&ahhead.station.chan[0],"%d",trhdr[7]);
   
   strcpy(&ahhead.station.stype[0], "PASSCAL");
   xs = (float)trhdr[35] ;   /* scale factor */
    if(xs < 0. )  xs = -1./xs ;    
	cptr = (unsigned char *)&llon ;
	cptr2 = (unsigned char *)&trhdr[40] ;
	for(i=0 ; i<4 ; i++,cptr++,cptr2++)  *cptr = *cptr2 ;
    xlon = xs*(float)llon ;
	cptr = (unsigned char *)&llat ;
	cptr2 = (unsigned char *)&trhdr[42] ;
	for(i=0 ; i<4 ; i++,cptr++,cptr2++)  *cptr = *cptr2 ;
    xlat = xs*(float)llat ;
    if(trhdr[44] = 1)   /* check position flag  */
    {
		convlat(&xlat,&xlon) ;   /* convert x-y to lat long */
    }
    else
    {
		xlat = xlat/3600. ;   /* positions in seconds of arc */
		xlon = xlon/3600. ;
    }
    ahhead.station.slon = xlon ;
    ahhead.station.slat = xlat ;
   ahhead.station.elev = 0.;
   ahhead.station.DS = 1.;
   ahhead.station.A0 = 0.; 

   /* now the event information */
   ahhead.event.ot.yr = trhdr[104];    /* the year of the trigger event */
   yearday = trhdr[105];
   for(iyear=1; yearday > day_tab[leap][iyear]; iyear++)yearday -= day_tab[leap][iyear];
   ahhead.event.ot.mo = iyear;
   ahhead.event.ot.day =  yearday;
   ahhead.event.ot.hr = trhdr[106];
   ahhead.event.ot.mn = trhdr[107];
   ahhead.event.ot.sec = (float) trhdr[108] + (float) trhdr[109]/1000.0;
	cptr = (unsigned char *)&llon ;
	cptr2 = (unsigned char *)&trhdr[36] ;
	for(i=0 ; i<4 ; i++,cptr++,cptr2++)  *cptr = *cptr2 ;
    xlon = xs*(float)llon ;
	cptr = (unsigned char *)&llat ;
	cptr2 = (unsigned char *)&trhdr[38] ;
	for(i=0 ; i<4 ; i++,cptr++,cptr2++)  *cptr = *cptr2 ;
    xlat = xs*(float)llat ;
    if(trhdr[44] = 1)   /* check position flag  */
    {
		convlat(&xlat,&xlon) ;   /* convert x-y to lat long */
    }
    else
    {
		xlat = xlat/3600. ;   /* positions in seconds of arc */
		xlon = xlon/3600. ;
    }
	ahhead.event.lon = xlon ;
    ahhead.event.lat = xlat ;
   
/* PREPARE FOR OUTPUT    */
   mkdir("./ah",0777);
   strcpy(FileOUT, "./ah/");
   strcat(FileOUT, FileIN);
   strcat(FileOUT, ".ah");
   if( (fpout = fopen(FileOUT,"wb" )) == NULL ){
      printf("\n Can't open %s for output",FileOUT);
      fclose(fpin);
      fclose(fpout);
      exit(1);
   }
   logger("SEGY;",&ahhead);
    /*  get scale factor for data */
    scale = 1. ;   /* default scale factor */
    gain = trhdr[60] ;   /* amplifier gain */
    cptr = (unsigned char *)&trhdr[110] ;  /* scale factor */
    cptr1 = (unsigned char *)&scale ;
    for(i=0 ; i<4 ; i++,cptr++,cptr1++) *cptr1 = *cptr ;
    if((gain == 0) || (scale == 0.))
    {
	printf("\nERROR in scale factor gain = %d scale= %f",gain,scale) ;
	printf("\n  Scale set to 1") ;
	scale = 1. ;
    }
    else 
    {
	scale = scale/(float)gain ;
    }

   /* now go get the trace data */ 

   if (( datac = calloc ( (unsigned) (ahhead.record.ndata),8)) == NULL){
      printf(" Error allocating space in %s \n",progname);
      exit(-2);
   }
   dout= data = (float *) datac;
   if (trhdr[102] == 1){            /* 32 bit integer data */

      for ( i=0; i<num_sam; i++){
         cptr = (unsigned char *) &int_hold;
         for (j=0; j<4; j++,cptr++){ 
            *cptr = fgetc(fpin);
         }
         *data = (float) int_hold;
	 *data = *data * scale ;
	 data++;
      }
   }else{                         /* 16 bit integer data */
      for ( i=0; i < num_sam; i++){
         cptr = (unsigned char *) &short_hold;
         *cptr = fgetc(fpin);
	 cptr++;
         *cptr = fgetc(fpin);
	 *data = (float) short_hold;
	 *data = *data * scale ;
	 data++;
      }
   }
   
   maxamp(&ahhead,( char *)dout);   
   puthead(&ahhead,fpout);
   putdata(&ahhead, (char *)dout, fpout);  
   cfree(datac);
   fclose(fpin);
   fclose(fpout);
}
}
 
/*********************************************************************/
/*		convlat(&xlat,&xlon)			             */
/*********************************************************************/
/*	Subroutine to convert x-y coordinates to lat long
	assume a spherical earth simple model
	 xlat is y coordinate on input returned as latitude
	 xlon is x coordinate on input returned as longitude

	lat positive to north
	long positive to east
*/
convlat(xlat,xlon)
float	*xlat, *xlon ;
{
	double 	d, D, L, mpd, tpi ;
	int		iflg ;

	iflg = 1 ;
	if(*xlon < 0.) iflg = -1 ;

	tpi =  3.1415927 ;
	mpd = (6080.2*60*.9144)/(3) ;   /* meters per degree lat */
	d = (double)((*xlat * *xlat) + (*xlon * *xlon)) ;
	d = sqrt(d) ;
	D = d/mpd ;    /* total distance */
	
	*xlat = *xlat/mpd ;        /* get latitude */

	d = cos(D * tpi/180.) ;
	L = cos(*xlat * tpi/180.) ;

	D = acos(d/L) ;    /* get longitude */
	D = (double)iflg * D ;  /* set direction */
	*xlon = (float)(D * 180./tpi) ;
	return ;
}
